/*
 * @(#)BasicCtrl.java		99/01/16
 *
 * Copyright (c) 1999 Noriki Amano. All Rights Reserved.
 *
 * This program was developed by Noriki Amano at JAIST 
 * as a part of a class library, titled "DAS".
 * All the programs of the language is developed by Noriki 
 * Amano and all rights are reserved by Noriki Amano.
 *
 */

import java.io.*;
import java.util.*;
import java.lang.*;
import java.lang.reflect.*;

/**
 *
 *
 * @author  Noriki Amano
 * @version 1.0, 99/01/16
 * @see
 * @since   JDK1.1.5
 */

public class BasicCtrl implements Serializable {
  public AdaptableMethod[] amTbl;

  private int IDX = 0;
  private int ENV = 0;
  private Object OBJ = null;
  private String DISPATCH	= "dispatch";
  private String LAST_INDEX	= "_";
  private String DOC_PREFIX	= "$DOC_";
  private String COND_PREFIX	= "$COND_";
  private String CODE_PREFIX	= "$CODE_";
  private String DEFAULT_STRATEGY = null;

  private String currentStrategyName	= null;
  private Method currentStrategyMethod	= null;

  public BasicCtrl() {
    this(200, "Default Strategy");
  }

  public BasicCtrl(int size, String strategyName) {
    Class amClass = (new AdaptableMethod()).getClass();
    amTbl = (AdaptableMethod[])Array.newInstance(amClass, size);
    tableNullClear(amTbl);
    DEFAULT_STRATEGY = strategyName;
  }

  public void tableNullClear(AdaptableMethod[] am) {
    for (int i = 0; i < am.length; i++) {
      am[i] = new AdaptableMethod();
    }
  }

  public Object adaptableCtrl(Object obj, String methodName, Object[] argObjects) {

    if (!methodSearch(obj, methodName)) {
      methodAppend(obj, methodName);
    }
    Method actualMethod = methodDispatch(argObjects[ENV], methodName, argObjects);
    if (actualMethod == null) {
      return(null);
    }
    return(Invocation.methodInvoke(actualMethod, obj, argObjects));
  }

  public boolean methodSearch(Object obj, String methodName) {
    String className = obj.getClass().getName();
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      String clsName = amTbl[i].getObject().getClass().getName();
      if (className.equals(clsName) 
      && methodName.equals(amTbl[i].getMethodName())) {
	return(true);
      }
    }
    return(false);
  }

  public void methodAppend(Object obj, String methodName) {
    Class cls   = obj.getClass();
    Field[]  ff = cls.getFields();
    Method[] mm = cls.getMethods();
    mm = methodJudge(mm, methodName);

    for (int i = 0; i < mm.length && IDX < amTbl.length; i++) {
      if (mm[i] == null) { 
	break;
      }
      String name = mm[i].getName();
      if (name.startsWith(COND_PREFIX)) {
	amTbl[IDX].setDocument(documentGet(obj, ff, name));
	amTbl[IDX].setObject(obj);
	amTbl[IDX].setMethodName(methodName);
	amTbl[IDX].setCondMethod(mm[i]);
	name = name.substring(COND_PREFIX.length(), name.length());
	name = CODE_PREFIX.concat(name);
	amTbl[IDX].setCodeMethod(methodGet(mm, name));
	IDX++;
      } 
    }
  }

  public Method methodGet(Method[] mm, String methodName) {

    for (int i = 0; i < mm.length; i++) {
      if (mm[i] == null) { break; }
      if (methodName.equals(mm[i].getName())) {
	return(mm[i]);
      }
    }
    System.out.println("Error --- actual method not found!");
    return(null);
  }

  public Method[] methodJudge(Method[] mm, String methodName) {
    Method[] mTbl = new Method[mm.length];
    for (int i = 0; i < mm.length; i++) {
      mTbl[i] = null;
    }
    int cnt = 0;
    int prefixLen = COND_PREFIX.length();
    for (int i = 0; i < mm.length; i++) {
      String name = mm[i].getName();
      if (name.startsWith(COND_PREFIX)
      ||  name.startsWith(CODE_PREFIX)) {
        int lastIdx = name.lastIndexOf(LAST_INDEX);
        name = name.substring(prefixLen, lastIdx);
        if (methodName.equals(name)) {
  	  mTbl[cnt++] = mm[i];
        }
      }
    }
    return(mTbl);
  }

  public String documentGet(Object obj, Field[] ff, String methodName) {
    int cnt = 0;
    for (int i = 0; i < ff.length; i++) {
      String name = ff[i].getName();
      if (name.startsWith(DOC_PREFIX)) {
        int lastIdx = name.length();
	int prefixLen = DOC_PREFIX.length();
        name = name.substring(prefixLen, lastIdx);
	lastIdx = methodName.length();
	prefixLen = COND_PREFIX.length();
	String mName = methodName.substring(prefixLen, lastIdx);
        if (mName.equals(name)) {
	  try {
	    return((String)ff[i].get(obj));
	  } catch (IllegalAccessException e) {
	    e.printStackTrace();
	  }
        }
      }
    }
    return(null);
  }

  public Method strategyMethodGet(String strategyName) {
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (amTbl[i].getMethodName().equals(DISPATCH)
      &&  amTbl[i].getDocument().equals(strategyName)) {
	return(amTbl[i].getCodeMethod());
      }
    }
    System.out.println("Error --- Default Strategy Method not found!");
    return(null);
  }

  public Object strategyObjectGet(String strategyName) {
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (amTbl[i].getMethodName().equals(DISPATCH)
      &&  amTbl[i].getDocument().equals(strategyName)) {
	return(amTbl[i].getObject());
      }
    }
    System.out.println("Error --- Default Strategy Object not found!");
    return(null);
  }

  public Method methodDispatch(Object obj, String methodName, Object[] args) {
    String strategy = ((StandardEnv)obj).getStrategy();
    if (methodName.equals(DISPATCH) 
    && currentStrategyName == strategy) {
      return(currentStrategyMethod);
    }
    Method mth	= null;
    Object arg1	= obj;
    Object arg2	= methodName;
    Object targetObj = OBJ;
    StandardEnv tempEnv = new StandardEnv(strategy);
    if (currentStrategyName == null) {
      arg1 = tempEnv;
      mth  = strategyMethodGet(DEFAULT_STRATEGY);
      targetObj = strategyObjectGet(DEFAULT_STRATEGY);
    } else {
      if (methodName.equals(DISPATCH)) {
	arg1 = tempEnv;
	mth  = strategyMethodGet(DEFAULT_STRATEGY);
	targetObj = strategyObjectGet(DEFAULT_STRATEGY);
      } else {
	mth = currentStrategyMethod;
      }
    }
    Object[] argObjects = {arg1, arg2, args};
    mth = (Method)Invocation.methodInvoke(mth, targetObj, argObjects);
    if (methodName.equals(DISPATCH) && mth == null) {
      System.out.println("Error --- Dispatch Strategy not found!");
    }
    if (methodName.equals(DISPATCH)) {
      currentStrategyName   = strategy;
      currentStrategyMethod = mth;

      for (int i = 0; amTbl[i] != null && i < amTbl.length; i++) {
	if (mth == amTbl[i].getCodeMethod()) {
	  OBJ = amTbl[i].getObject();
	  break;
	}
      }

    }
    return(mth);
  }

}
