/*
 * @(#)SpecialStrategy.java		99/01/16
 *
 * Copyright (c) 1999 Noriki Amano. All Rights Reserved.
 *
 * This program was developed by Noriki Amano at JAIST 
 * as a part of a class library, titled "DAS".
 * All the programs of the language is developed by Noriki 
 * Amano and all rights are reserved by Noriki Amano.
 *
 */

import java.io.*;
import java.util.*;
import java.lang.*;
import java.lang.reflect.*;

/**
 *
 *
 * @author  Noriki Amano
 * @version 1.0, 99/01/16
 * @see
 * @since   JDK1.1.5
 */

public class SpecialStrategy extends StandardStrategy {
  protected String UserStrategyResult = null;

  private int MAXIMUM = -1;
  private int MINIMUM = -1000;
  private String PositiveReturnType = null;
  private String NegativeReturnType = null;
  private String UserLikeReturnType = null;

  private SpecialStrategy THIS = null;

  public SpecialStrategy(AdaptableMethod[] am) {
    this(am, "EvalArgs", "EvalArgs", "EvalArgs");
  }

  public SpecialStrategy(AdaptableMethod[] am, String positiveType,
			 String negativeType, String userLikeType) {
    super(am);
    amTbl = am;
    THIS = this;
    PositiveReturnType = positiveType;
    NegativeReturnType = negativeType;
    UserLikeReturnType = userLikeType;
  }

// Positive Priority Strategy (Document Part)
  public String $DOC_dispatch_4 = "Positive Priority Strategy";

// Positive Priority Strategy (Condition Part)
  public EvalArgs $COND_dispatch_4(StandardEnv obj) {
    System.out.println($DOC_dispatch_4 + " condition is evaluated!");
    EvalArgs eArgs = new EvalArgs(obj);
    eArgs.element  = new String[] {"strategy"};
    eArgs.bools	   = new boolean[] {obj.getStrategy() == "positive"};
    return(eArgs);
  }
// Positive Priority Strategy (Code Part)
  public Method $CODE_dispatch_4(StandardEnv obj, String methodName, Object[] args) {
    System.out.println($DOC_dispatch_4 + " codes are executed!");
    if (methodName.equals("dispatch")) { 
      return(null); 
    }
    int max = MAXIMUM;
    Method tempMethod   = null;
    Method returnMethod = null;
    Object[] argObjects = {obj};
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (methodName.equals(amTbl[i].getMethodName())) {
        Method mth = amTbl[i].getCondMethod();
	if (TypeCheck.condMethodCheck(obj, mth, PositiveReturnType)) {
	  EvalArgs ea = (EvalArgs)Invocation.methodInvoke(mth, 
							  amTbl[i].getObject(), 
							  argObjects);
	  int sum = ea.positiveSum(obj);
	  if (TypeCheck.codeMethodCheck(amTbl[i].getCodeMethod(), methodName, args)) {
	    tempMethod = amTbl[i].getCodeMethod();
	    if (max <= sum) {
	      max = sum;
	      returnMethod = tempMethod;
	    }
	  }
	}
      }
    }
    if (returnMethod == null) {
      return(tempMethod);
    }
    return(returnMethod);
  }

// Negative Priority Strategy (Document Part)
  public String $DOC_dispatch_5 = "Negative Priority Strategy";

// Negative Priority Strategy (Condition Part)
  public EvalArgs $COND_dispatch_5(StandardEnv obj) {
    System.out.println($DOC_dispatch_5 + " condition is evaluated!");
    EvalArgs eArgs = new EvalArgs(obj);
    eArgs.element  = new String[] {"strategy"};
    eArgs.bools	   = new boolean[] {obj.getStrategy() == "negative"};
    return(eArgs);
  }
// Negative Priority Strategy (Code Part)
  public Method $CODE_dispatch_5(StandardEnv obj, String methodName, Object[] args) {
    System.out.println($DOC_dispatch_5 + " codes are executed!");
    if (methodName.equals("dispatch")) { 
      return(null); 
    }
    int min = MINIMUM;
    Method tempMethod   = null;
    Method returnMethod = null;
    Object[] argObjects = {obj};
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (methodName.equals(amTbl[i].getMethodName())) {
        Method mth = amTbl[i].getCondMethod();
	if (TypeCheck.condMethodCheck(obj, mth, NegativeReturnType)) {
	  EvalArgs ea = (EvalArgs)Invocation.methodInvoke(mth, 
							  amTbl[i].getObject(), 
							  argObjects);
	  int sum = ea.negativeSum(obj);
	  if (TypeCheck.codeMethodCheck(amTbl[i].getCodeMethod(), methodName, args)) {
	    tempMethod = amTbl[i].getCodeMethod();
	    if (min <= sum) {
	      min = sum;
	      returnMethod = tempMethod;
	    }
	  }
	}
      }
    }
    if (returnMethod == null) {
      return(tempMethod);
    }
    return(returnMethod);
  }

// UserLike Strategy (Document Part)
  public String $DOC_dispatch_6 = "UserLike Strategy";

// UserLike Strategy (Condition Part)
  public EvalArgs $COND_dispatch_6(StandardEnv obj) {
    System.out.println($DOC_dispatch_6 + " condition is evaluated!");
    EvalArgs eArgs = new EvalArgs(obj);
    eArgs.element  = new String[] {"strategy"};
    eArgs.bools	   = new boolean[] {obj.getStrategy() == "user-like"};
    return(eArgs);
  }

// UserLike Strategy (Code Part)
  public Method $CODE_dispatch_6(StandardEnv obj, String methodName, Object[] args) {
    System.out.println($DOC_dispatch_6 + " codes are executed!");
    if (methodName.equals("dispatch")) { 
      return(null); 
    }
    MethodList ml = new MethodList(THIS);
    Object[] argObjects = {obj};
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (methodName.equals(amTbl[i].getMethodName())) {
        Method mth = amTbl[i].getCondMethod();
	if (TypeCheck.condMethodCheck(obj, mth, UserLikeReturnType)
	&& TypeCheck.codeMethodCheck(amTbl[i].getCodeMethod(), methodName, args)) {
	  ml.methodList.addItem(amTbl[i].getDocument());
	}
      }
    }
    UserStrategyResult = null;
    ml.show();
//    for (; UserStrategyResult == null; ) {}
//    ml.dispose();
    if (!UserStrategyResult.equals("UserStrategyCancel")) {
      for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
        String methodDoc = amTbl[i].getDocument();
        if (UserStrategyResult.equals(methodDoc)) {
	  return(amTbl[i].getCodeMethod());
	}
      }
    }
    return(null);
  }

}
