/*
 * @(#)StandardEnv.java		99/01/16
 *
 * Copyright (c) 1999 Noriki Amano. All Rights Reserved.
 *
 * This program was developed by Noriki Amano at JAIST 
 * as a part of a class library, titled "DAS".
 * All the programs of the language is developed by Noriki 
 * Amano and all rights are reserved by Noriki Amano.
 *
 */

import java.io.*;
import java.util.*;
import java.lang.*;

/**
 *
 *
 * @author  Noriki Amano
 * @version 1.0, 99/01/16
 * @see
 * @since   JDK1.1.5
 */

public class StandardEnv extends Environment implements Serializable {
  private static String[] defaultPriorList = {	"power",
						"battery",
						"memory", 
						"disk",
						"display"	};
  public String[] priorityList	= null;
  public int[] priorityNumber	= null;

  private String strategy = "default";

  public StandardEnv() {
    this("default", defaultPriorList);
  }

  public StandardEnv(String Strategy) {
    this(Strategy, defaultPriorList);
  }

  public StandardEnv(String[] pList) {
    this("default", pList);
  }

  public StandardEnv(String strategyName, String[] pList) {
    strategy = strategyName;
    priorityList = pList;
    setPriorNumber();    
  }

  public void setPriorNumber() {
    priorityNumber = new int[priorityList.length];
    int j = priorityList.length - 1;
    for (int i = 0; i < priorityList.length && j >= 0; i++, j--) {
      priorityNumber[j] = power(i);
    }
  }

  public int getPriorNumber(String elementName) {
    for (int i = 0; i < priorityList.length; i++) {
      if (priorityList[i] == elementName) return(priorityNumber[i]);
    }
    return(0); // Error case!
  }

  public int power(int cnt) {
    int pow = 1;
    if (cnt == 0) return(pow);
    for (int i = 0; i < cnt; i++) {
      pow *= 2;
    }
    return(pow);
  }

  public void setStrategy(String value) { strategy = value; }
  public String getStrategy()	{ return(strategy); }

  public String getDisplay() { 
    int width  = getDisplayWidth();
    int height = getDisplayHeight();
    if (width >= 1024 && height >= 768) {	// 1024 * 800
      return("LARGE");
    } else if (width >= 800 && height >= 600) {	// 800 * 600
      return("MIDDLE");
    } else {					// 600 * 480
      return("SMALL");
    }
  }
  public String getBattery() { 
    int battery = getBatteryRemainder();
    if (battery >= 90) {	// Battery Remainder more than 90%
      return("LARGE");
    } else if (battery >= 50) {	// Battery Remainder more than 50%
      return("MIDDLE");
    } else {
      return("SMALL");		// Battery Remainder less than 50%
    }
  }
  public String getMemory() { 
    int memory = getFreeMemory();
    memory = memory/1024/1024;
    if (memory >= 50) {		// Free Memory Size more than 50MB
      return("LARGE");
    } else if (memory >= 20) {	// Free Memory Size more than 20MB
      return("MIDDLE");
    } else {
      return("SMALL");		// Free Memory Size less than 20MB
    }
  }
  public String getPower() { 
    int power = getPowerSource();
    if (power == 0) {
      return("BATTERY");
    } else {
      return("AC-POWER");
    }
  }
  public String getDisk() { 
    int disk = getDiskFreeSpace();
    disk = disk/1024/1024;
    if (disk >= 1000) {		// Disk Free Space more than 1GB
      return("LARGE");
    } else if (disk >= 500) {	// Disk Free Space more than 500MB
      return("MIDDLE");
    } else {			// Disk Free Space less than 500MB
      return("SMALL");
    }
  }
  public String getHostName()	{ return(localHostName); }

}
