/*
 * @(#)StandardStrategy.java		99/01/16
 *
 * Copyright (c) 1999 Noriki Amano. All Rights Reserved.
 *
 * This program was developed by Noriki Amano at JAIST 
 * as a part of a class library, titled "DAS".
 * All the programs of the language is developed by Noriki 
 * Amano and all rights are reserved by Noriki Amano.
 *
 */

import java.io.*;
import java.util.*;
import java.lang.*;
import java.lang.reflect.*;

/**
 *
 *
 * @author  Noriki Amano
 * @version 1.0, 99/01/16
 * @see
 * @since   JDK1.1.5
 */

public class StandardStrategy extends TypeCheck implements Serializable {
  protected AdaptableMethod[] amTbl;

  private String DISPATCH = "dispatch";
  private String DefaultReturnType = null;
  private String OrReturnType	  = null;
  private String AndReturnType	  = null;

  public StandardStrategy() {}

  public StandardStrategy(AdaptableMethod[] am) {
    this(am, "EvalArgs", "EvalArgs", "EvalArgs");
  }

  public StandardStrategy(AdaptableMethod[] am, String defaultType,
			  String orType, String andType) {
    amTbl = am;
    DefaultReturnType = defaultType;
    OrReturnType      = orType;
    AndReturnType     = andType;
  }

// Default Strategy (Document Part)
  public String $DOC_dispatch_1 = "Default Strategy";

// Default Strategy (Condition Part)
  public EvalArgs $COND_dispatch_1(StandardEnv obj) {
    System.out.println($DOC_dispatch_1 + " condition is evaluated!");
    EvalArgs eArgs = new EvalArgs(obj);
    eArgs.element  = new String[] {"strategy"};
    eArgs.bools	   = new boolean[] {obj.getStrategy() == "default"};
    return(eArgs);
  }
// Default Strategy (Code Part)
  public Method $CODE_dispatch_1(StandardEnv obj, String methodName, Object[] args) {
    System.out.println($DOC_dispatch_1 + " codes are executed!");
    Method returnMethod = null;
    Object[] argObjects = {obj};
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (methodName.equals(amTbl[i].getMethodName())) {
        Method mth = amTbl[i].getCondMethod();
	if (TypeCheck.condMethodCheck(obj, mth, DefaultReturnType)) {
	  EvalArgs ea = (EvalArgs)Invocation.methodInvoke(mth,
							  amTbl[i].getObject(), 
							  argObjects);
	  if (TypeCheck.codeMethodCheck(amTbl[i].getCodeMethod(), methodName, args)) {
	    returnMethod = amTbl[i].getCodeMethod();
	    if (ea.andEval()) {
	      return(returnMethod);
	    }
	  }
	}
      }
    }
    return(returnMethod);
  }

// OR Strategy (Document Part)
  public String $DOC_dispatch_2 = "OR Strategy";

// OR Strategy (Condition Part)
  public EvalArgs $COND_dispatch_2(StandardEnv obj) {
    System.out.println($DOC_dispatch_2 + " condition is evaluated!");
    EvalArgs eArgs = new EvalArgs(obj);
    eArgs.element  = new String[] {"strategy"};
    eArgs.bools	   = new boolean[] {obj.getStrategy() == "or"};
    return(eArgs);
  }
// OR Strategy (Code Part)
  public Method $CODE_dispatch_2(StandardEnv obj, String methodName, Object[] args) {
    System.out.println($DOC_dispatch_2 +" codes are executed!");
    if (methodName.equals("dispatch")) { 
      return(null); 
    }
    Object[] argObjects = {obj};
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (methodName.equals(amTbl[i].getMethodName())) {
        Method mth = amTbl[i].getCondMethod();
	if (TypeCheck.condMethodCheck(obj, mth, OrReturnType)) {
	  EvalArgs ea = (EvalArgs)Invocation.methodInvoke(mth, 
							  amTbl[i].getObject(), 
							  argObjects);
	  if (ea.orEval() 
	  && TypeCheck.codeMethodCheck(amTbl[i].getCodeMethod(), methodName, args)) {
	    return(amTbl[i].getCodeMethod());
	  }
	}
      }
    }
    return(null);
  }

// AND Strategy (Document Part)
  public String $DOC_dispatch_3 = "AND Strategy";

// AND Strategy (Condition Part)
  public EvalArgs $COND_dispatch_3(StandardEnv obj) {
    System.out.println($DOC_dispatch_3 + " condition is evaluated!");
    EvalArgs eArgs = new EvalArgs(obj);
    eArgs.element  = new String[] {"strategy"};
    eArgs.bools	   = new boolean[] {obj.getStrategy() == "and"};
    return(eArgs);
  }
// AND Strategy (Code Part)
  public Method $CODE_dispatch_3(StandardEnv obj, String methodName, Object[] args) {
    System.out.println($DOC_dispatch_3 +" codes are executed!");
    if (methodName.equals("dispatch")) { 
      return(null); 
    }
    Object[] argObjects = {obj};
    for (int i = 0; amTbl[i].getObject() != null && i < amTbl.length; i++) {
      if (methodName.equals(amTbl[i].getMethodName())) {
        Method mth = amTbl[i].getCondMethod();
	if (TypeCheck.condMethodCheck(obj, mth, AndReturnType)) {
	  EvalArgs ea = (EvalArgs)Invocation.methodInvoke(mth, 
							  amTbl[i].getObject(), 
							  argObjects);
	  if (ea.andEval() 
	  && TypeCheck.codeMethodCheck(amTbl[i].getCodeMethod(), methodName, args)) {
	    return(amTbl[i].getCodeMethod());
	  }
	}
      }
    }
    return(null);
  }

}
